#coding: utf-8

from south.db import db
from django.db import models
from robokassa.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'SuccessNotification'
        db.create_table('robokassa_successnotification', (
            ('id', orm['robokassa.SuccessNotification:id']),
            ('InvId', orm['robokassa.SuccessNotification:InvId']),
            ('OutSum', orm['robokassa.SuccessNotification:OutSum']),
            ('created_at', orm['robokassa.SuccessNotification:created_at']),
        ))
        db.send_create_signal('robokassa', ['SuccessNotification'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'SuccessNotification'
        db.delete_table('robokassa_successnotification')
        
    
    
    models = {
        'robokassa.successnotification': {
            'InvId': ('django.db.models.fields.IntegerField', [], {'db_index': 'True'}),
            'OutSum': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['robokassa']



#coding: utf-8

from datetime import datetime
from django.db import models

class SuccessNotification(models.Model):
    InvId = models.IntegerField(u'Номер заказа', db_index=True)
    OutSum = models.CharField(u'Сумма', max_length=15)

    created_at = models.DateTimeField(u'Дата и время получения уведомления', default = datetime.now)

    def __unicode__(self):
        return u'#%d: %s (%s)' % (self.InvId, self.OutSum, self.created_at)

    class Meta:
        verbose_name = u'Уведомление об успешном платеже'
        verbose_name_plural = u'Уведомления об успешных платежах (ROBOKASSA)'



#coding: utf-8

from django.conf import settings

# обязательные параметры - реквизиты магазина
LOGIN = settings.ROBOKASSA_LOGIN
PASSWORD1 = settings.ROBOKASSA_PASSWORD1
PASSWORD2 = getattr(settings, 'ROBOKASSA_PASSWORD2', None)

# использовать ли метод POST при приеме результатов
USE_POST = getattr(settings, 'ROBOKASSA_USE_POST', True)

# требовать предварительного уведомления на ResultURL
STRICT_CHECK = getattr(settings, 'ROBOKASSA_STRICT_CHECK', True)

# тестовый режим
TEST_MODE = getattr(settings, 'ROBOKASSA_TEST_MODE', False)

# url, по которому будет идти отправка форм
FORM_TARGET = u'https://merchant.roboxchange.com/Index.aspx'
if TEST_MODE:
    FORM_TARGET = u'http://test.robokassa.ru/Index.aspx'

# список пользовательских параметров ("shp" к ним приписывать не нужно)
EXTRA_PARAMS = sorted(getattr(settings, 'ROBOKASSA_EXTRA_PARAMS', []))



#coding: utf-8
from unittest import TestCase
from django.test import TestCase as DjangoTestCase
from robokassa.forms import RobokassaForm, ResultURLForm
from robokassa.conf import LOGIN, PASSWORD1, PASSWORD2

class RobokassaFormTest(TestCase):

    def setUp(self):
        self.form = RobokassaForm(initial = {
                       'OutSum': 100.00,
                       'InvId': 58,
                       'Desc' : u'Холодильник "Бирюса"',
                       'Email' : 'vasia@example.com'
                    })

    def testSignature(self):
        self.assertEqual(self.form._get_signature_string(),
                         '%s:100.0:58:%s:shpparam1=None:shpparam2=None' % (LOGIN, PASSWORD1))
        self.assertEqual(len(self.form.fields['SignatureValue'].initial), 32)

    def testSignatureMissingParams(self):
        form = RobokassaForm(initial = {'InvId': 5})
        self.assertEqual(form._get_signature_string(),
                         '%s::5:%s:shpparam1=None:shpparam2=None' % (LOGIN, PASSWORD1))

    def testRedirectUrl(self):
        url = "https://merchant.roboxchange.com/Index.aspx?MrchLogin=test_login&OutSum=100.0&InvId=58&Desc=%D5%EE%EB%EE%E4%E8%EB%FC%ED%E8%EA+%22%C1%E8%F0%FE%F1%E0%22&SignatureValue=0EC23BE40003640B35EC07F6615FFB57&Email=vasia%40example.com&shpparam1=None&shpparam2=None"
        self.assertEqual(self.form.get_redirect_url(), url)


class RobokassaFormExtraTest(TestCase):
    def testExtra(self):
        form = RobokassaForm(initial={
                                'InvId': 58,
                                'OutSum': 100,
                                'param1': 'value1',
                                'param2': 'value2'
                            })
        self.assertEqual(form._get_signature_string(),
                         '%s:100:58:%s:shpparam1=value1:shpparam2=value2' % (LOGIN, PASSWORD1))


class ResultURLTest(DjangoTestCase):

    def testFormExtra(self):
        form = ResultURLForm({
                'OutSum': '100',
                'InvId': '58',
                'SignatureValue': 'B2111A06F6B7A1E090D38367BF7032D9',
                'shpparam1': 'Vasia',
                'shpparam2': 'None',
             })
        self.assertTrue(form.is_valid())
        self.assertEqual(form._get_signature_string(),
                         '100:58:%s:shpparam1=Vasia:shpparam2=None' % (PASSWORD2))
        self.assertEqual(form.extra_params(), {'param1': 'Vasia', 'param2': 'None'})


    def testFormValid(self):

        self.assertTrue(ResultURLForm({
                'OutSum': '100',
                'InvId': '58',
                'SignatureValue': '877D3BAF8381F70E56638C3BC82580C5',
                'shpparam1': 'None',
                'shpparam2': 'None',
             }).is_valid())

        self.assertFalse(ResultURLForm({
                'OutSum': '101',
                'InvId': '58',
                'SignatureValue': '877D3BAF8381F70E56638C3BC82580C5',
                'shpparam1': 'None',
                'shpparam2': 'None',
             }).is_valid())

        self.assertFalse(ResultURLForm({
                'OutSum': '100',
                'InvId': '58',
                'SignatureValue': '877D3BAF8381F70E56638C3BC82580C5',
                'shpparam1': 'Vasia',
                'shpparam2': 'None',
             }).is_valid())



#coding: utf-8
from django.conf.urls.defaults import *

urlpatterns = patterns('robokassa.views',
    url(
          r'^result/$',
          'receive_result',
          name='robokassa_result'
    ),
    url(
          r'^success/$',
          'success',
          name='robokassa_success'
    ),
    url(
          r'^fail/$',
          'fail',
          name='robokassa_fail'
    ),
)



#coding: utf-8

from hashlib import md5
from urllib import urlencode
from django import forms

from robokassa.conf import LOGIN, PASSWORD1, PASSWORD2
from robokassa.conf import STRICT_CHECK, FORM_TARGET, EXTRA_PARAMS
from robokassa.models import SuccessNotification

class BaseRobokassaForm(forms.Form):

    def __init__(self, *args, **kwargs):
        super(BaseRobokassaForm, self).__init__(*args, **kwargs)
        # создаем дополнительные поля
        for key in EXTRA_PARAMS:
            self.fields['shp'+key] = forms.CharField(required=False)
            if 'initial' in kwargs:
                self.fields['shp'+key].initial = kwargs['initial'].get(key, 'None')

    def _append_extra_part(self, standard_part, value_func):
        extra_part = ":".join(["%s=%s" % ('shp'+key, value_func('shp' + key)) for key in EXTRA_PARAMS])
        if extra_part:
            return ':'.join([standard_part, extra_part])
        return standard_part

    def _get_signature(self):
        return md5(self._get_signature_string()).hexdigest().upper()

    def _get_signature_string(self):
        raise NotImplementedError


class RobokassaForm(BaseRobokassaForm):

    # login магазина в обменном пункте
    MrchLogin = forms.CharField(max_length=20, initial = LOGIN)

    # требуемая к получению сумма
    OutSum = forms.DecimalField(min_value=0, max_digits=20, decimal_places=2, required=False)

    # номер счета в магазине (должен быть уникальным для магазина)
    InvId = forms.IntegerField(min_value=0, required=False)

    # описание покупки
    Desc = forms.CharField(max_length=100, required=False)

    # контрольная сумма MD5
    SignatureValue = forms.CharField(max_length=32)

    # предлагаемая валюта платежа
    IncCurrLabel = forms.CharField(max_length = 10, required=False)

    # e-mail пользователя
    Email = forms.CharField(max_length=100, required=False)

    # язык общения с клиентом (en или ru)
    Culture = forms.CharField(max_length=10, required=False)

    # Параметр с URL'ом, на который форма должны быть отправлена.
    # Может пригодиться для использования в шаблоне.
    target = FORM_TARGET

    def __init__(self, *args, **kwargs):

        super(RobokassaForm, self).__init__(*args, **kwargs)

        # скрытый виджет по умолчанию
        for field in self.fields:
            self.fields[field].widget = forms.HiddenInput()

        self.fields['SignatureValue'].initial = self._get_signature()


    def get_redirect_url(self):
        """ Получить URL с GET-параметрами, соответствующими значениям полей в
        форме. Редирект на адрес, возвращаемый этим методом, эквивалентен
        ручной отправке формы методом GET.
        """
        def _initial(name, field):
            val = self.initial.get(name, field.initial)
            if not val:
                return val
            return unicode(val).encode('1251')

        fields = [(name, _initial(name, field))
                  for name, field in self.fields.items()
                  if _initial(name, field)
                 ]
        params = urlencode(fields)
        return self.target+'?'+params

    def _get_signature_string(self):
        def _val(name):
            value = self.initial[name] if name in self.initial else self.fields[name].initial
            if value is None:
                return ''
            return unicode(value)
        standard_part = ':'.join([_val('MrchLogin'), _val('OutSum'), _val('InvId'), PASSWORD1])
        return self._append_extra_part(standard_part, _val)



class ResultURLForm(BaseRobokassaForm):
    '''Форма для приема результатов и проверки контрольной суммы '''
    OutSum = forms.CharField(max_length=15)
    InvId = forms.IntegerField(min_value=0)
    SignatureValue = forms.CharField(max_length=32)

    def clean(self):
        if self.cleaned_data['SignatureValue'].upper() != self._get_signature():
#            print self._get_signature()
            raise forms.ValidationError(u'Ошибка в контрольной сумме')
        return self.cleaned_data

    def extra_params(self):
        extra = {}
        for param in EXTRA_PARAMS:
            if ('shp'+param) in self.cleaned_data:
                extra[param] = self.cleaned_data['shp'+param]
        return extra

    def _get_signature_string(self):
        _val = lambda name: unicode(self.cleaned_data[name])
        standard_part = ':'.join([_val('OutSum'), _val('InvId'), PASSWORD2])
        return self._append_extra_part(standard_part, _val)


class _RedirectPageForm(ResultURLForm):
    '''Форма для проверки контрольной суммы на страницах Success и Fail'''

    Culture = forms.CharField(max_length=10)

    def _get_signature_string(self):
        _val = lambda name: unicode(self.cleaned_data[name])
        standard_part = ':'.join([_val('OutSum'), _val('InvId'), PASSWORD1])
        return self._append_extra_part(standard_part, _val)


class FailRedirectForm(_RedirectPageForm):
    '''Форма для проверки контрольной суммы на странице Fail'''
    pass

class SuccessRedirectForm(_RedirectPageForm):
    """ Форма для обработки страницы Success с дополнительной защитой. Она
    проверяет, что ROBOKASSA предварительно уведомила систему о платеже,
    отправив запрос на ResultURL. """

    def clean(self):
        data = super(SuccessRedirectForm, self).clean()
        if STRICT_CHECK:
            if not SuccessNotification.objects.filter(InvId=data['InvId']):
                raise forms.ValidationError(u'От ROBOKASSA не было предварительного уведомления')
        return data



#!/usr/bin/env python
#coding: utf-8
from distutils.core import setup

import sys
reload(sys).setdefaultencoding("UTF-8")

setup(
    name='django-robokassa',
    version='0.9.1',
    author='Mikhail Korobov',
    author_email='kmike84@gmail.com',

    packages=['robokassa', 'robokassa.migrations'],

    url='http://bitbucket.org/kmike/django-robokassa/',
    download_url = 'http://bitbucket.org/kmike/django-robokassa/get/tip.zip',
    license = 'MIT license',
    description = u'Приложение для интеграции платежной системы ROBOKASSA в проекты на Django.'.encode('utf8'),
    long_description = open('README.rst').read().decode('utf8'),

    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Natural Language :: Russian',
    ),
)


#coding: utf-8
from django.dispatch import Signal

result_received = Signal(providing_args=["InvId", "OutSum"])
success_page_visited = Signal(providing_args=["InvId", "OutSum"])
fail_page_visited = Signal(providing_args=["InvId", "OutSum"])




#coding: utf-8

from django.http import HttpResponse
from django.views.generic.simple import direct_to_template

from robokassa.conf import USE_POST
from robokassa.forms import ResultURLForm, SuccessRedirectForm, FailRedirectForm
from robokassa.models import SuccessNotification
from robokassa.signals import result_received, success_page_visited, fail_page_visited

def receive_result(request):
    """ обработчик для ResultURL. """
    data = request.POST if USE_POST else request.GET
    form = ResultURLForm(data)
    if form.is_valid():
        id, sum = form.cleaned_data['InvId'], form.cleaned_data['OutSum']

        # сохраняем данные об успешном уведомлении в базе, чтобы
        # можно было выполнить дополнительную проверку на странице успешного
        # заказа
        notification = SuccessNotification.objects.create(InvId = id, OutSum = sum)

        # дополнительные действия с заказом (например, смену его статуса) можно
        # осуществить в обработчике сигнала robokassa.signals.result_received
        result_received.send(sender = notification, InvId = id, OutSum = sum,
                             extra = form.extra_params())

        return HttpResponse('OK%s' % id)
    return HttpResponse('error: bad signature')


def success(request, template_name='robokassa/success.html', extra_context=None,
            error_template_name = 'robokassa/error.html'):
    """ обработчик для SuccessURL """

    data = request.POST if USE_POST else request.GET
    form = SuccessRedirectForm(data)
    if form.is_valid():
        id, sum = form.cleaned_data['InvId'], form.cleaned_data['OutSum']

        # в случае, когда не используется строгая проверка, действия с заказом
        # можно осуществлять в обработчике сигнала robokassa.signals.success_page_visited
        success_page_visited.send(sender = form, InvId = id, OutSum = sum,
                                  extra = form.extra_params())

        context = {'InvId': id, 'OutSum': sum, 'form': form}
        context.update(form.extra_params())
        context.update(extra_context or {})
        return direct_to_template(request, template_name, extra_context=context)

    return direct_to_template(request, error_template_name, extra_context={'form': form})


def fail(request, template_name='robokassa/fail.html', extra_context=None,
         error_template_name = 'robokassa/error.html'):
    """ обработчик для FailURL """

    data = request.POST if USE_POST else request.GET
    form = FailRedirectForm(data)
    if form.is_valid():
        id, sum = form.cleaned_data['InvId'], form.cleaned_data['OutSum']

        # дополнительные действия с заказом (например, смену его статуса для
        # разблокировки товара на складе) можно осуществить в обработчике
        # сигнала robokassa.signals.fail_page_visited
        fail_page_visited.send(sender = form, InvId = id, OutSum = sum,
                               extra = form.extra_params())

        context = {'InvId': id, 'OutSum': sum, 'form': form}
        context.update(form.extra_params())
        context.update(extra_context or {})
        return direct_to_template(request, template_name, extra_context=context)

    return direct_to_template(request, error_template_name, extra_context={'form': form})

